@@ -39,16 +39,16 @@
 using namespace std;
 
 
-Controller::Controller(const std::string& hostId, size_t instance, std::shared_ptr<MetadataAdapter> meta)
-    : MessageReceiver(), hostId_(hostId), instance_(instance), active_(false), latency_(0), stream_(nullptr), decoder_(nullptr), player_(nullptr), meta_(meta),
+Controller::Controller(const ClientSettings& settings, std::unique_ptr<MetadataAdapter> meta)
+    : MessageReceiver(), settings_(settings), active_(false), stream_(nullptr), decoder_(nullptr), player_(nullptr), meta_(std::move(meta)),
       serverSettings_(nullptr), async_exception_(nullptr)
 {
 }
 
 
-void Controller::onException(ClientConnection* /*connection*/, shared_exception_ptr exception)
+void Controller::onException(ClientConnection* /*connection*/, std::exception_ptr exception)
 {
-    LOG(ERROR) << "Controller::onException: " << exception->what() << "\n";
+    LOG(ERROR) << "Controller::onException\n";
     async_exception_ = exception;
 }
 
@@ -60,18 +60,16 @@ void Controller::onMessageReceived(ClientConnection* /*connection*/, const msg::
     {
         if (stream_ && decoder_)
         {
-            auto* pcmChunk = new msg::PcmChunk(sampleFormat_, 0);
+            auto pcmChunk = make_unique<msg::PcmChunk>(sampleFormat_, 0);
             pcmChunk->deserialize(baseMessage, buffer);
-            // LOG(DEBUG) << "chunk: " << pcmChunk->payloadSize << ", sampleFormat: " << sampleFormat_.rate << "\n";
-            if (decoder_->decode(pcmChunk))
+            // LOG(DEBUG) << "chunk: " << pcmChunk->payloadSize << ", sampleFormat: " << sampleFormat_.getFormat() << "\n";
+            if (decoder_->decode(pcmChunk.get()))
             {
                 // TODO: do decoding in thread?
-                stream_->addChunk(pcmChunk);
+                stream_->addChunk(move(pcmChunk));
                 // LOG(DEBUG) << ", decoded: " << pcmChunk->payloadSize << ", Duration: " << pcmChunk->getDuration() << ", sec: " << pcmChunk->timestamp.sec <<
                 // ", usec: " << pcmChunk->timestamp.usec/1000 << ", type: " << pcmChunk->type << "\n";
             }
-            else
-                delete pcmChunk;
         }
     }
     else if (baseMessage.type == message_type::kTime)
@@ -121,20 +119,32 @@ void Controller::onMessageReceived(ClientConnection* /*connection*/, const msg::
             throw SnapException("codec not supported: \"" + headerChunk_->codec + "\"");
 
         sampleFormat_ = decoder_->setHeader(headerChunk_.get());
-        LOG(NOTICE) << TAG("state") << "sampleformat: " << sampleFormat_.rate << ":" << sampleFormat_.bits << ":" << sampleFormat_.channels << "\n";
+        LOG(NOTICE) << TAG("state") << "sampleformat: " << sampleFormat_.getFormat() << "\n";
 
-        stream_ = make_shared<Stream>(sampleFormat_);
-        stream_->setBufferLen(serverSettings_->getBufferMs() - latency_);
+        stream_ = make_shared<Stream>(sampleFormat_, settings_.player.sample_format);
+        stream_->setBufferLen(serverSettings_->getBufferMs() - settings_.player.latency);
 
+        const auto& pcm_device = settings_.player.pcm_device;
+        const auto& player_name = settings_.player.player_name;
+        player_ = nullptr;
 #ifdef HAS_ALSA
-        player_ = make_unique<AlsaPlayer>(pcmDevice_, stream_);
-#elif HAS_OPENSL
-        player_ = make_unique<OpenslPlayer>(pcmDevice_, stream_);
-#elif HAS_COREAUDIO
-        player_ = make_unique<CoreAudioPlayer>(pcmDevice_, stream_);
-#else
-        throw SnapException("No audio player support");
+        if (!player_ && (player_name.empty() || (player_name == "alsa")))
+            player_ = make_unique<AlsaPlayer>(pcm_device, stream_);
 #endif
+#ifdef HAS_OBOE
+        if (!player_ && (player_name.empty() || (player_name == "oboe")))
+            player_ = make_unique<OboePlayer>(pcm_device, stream_);
+#endif
+#ifdef HAS_OPENSL
+        if (!player_ && (player_name.empty() || (player_name == "opensl")))
+            player_ = make_unique<OpenslPlayer>(pcm_device, stream_);
+#endif
+#ifdef HAS_COREAUDIO
+        if (!player_ && (player_name.empty() || (player_name == "coreaudio")))
+            player_ = make_unique<CoreAudioPlayer>(pcm_device, stream_);
+#endif
+        if (!player_)
+            throw SnapException("No audio player support");
         player_->setVolume(serverSettings_->getVolume() / 100.);
         player_->setMute(serverSettings_->isMuted());
         player_->start();
@@ -149,16 +159,16 @@ void Controller::onMessageReceived(ClientConnection* /*connection*/, const msg::
         }
     }
 
-    if (baseMessage.type != message_type::kTime)
-        if (sendTimeSyncMessage(1000))
-            LOG(DEBUG) << "time sync onMessageReceived\n";
+    // if (baseMessage.type != message_type::kTime)
+    //     if (sendTimeSyncMessage(1000))
+    //         LOG(DEBUG) << "time sync onMessageReceived\n";
 }
 
 
-bool Controller::sendTimeSyncMessage(long after)
+bool Controller::sendTimeSyncMessage(const std::chrono::milliseconds& after)
 {
-    static long lastTimeSync(0);
-    long now = chronos::getTickCount();
+    static chronos::time_point_clk lastTimeSync(chronos::clk::now());
+    auto now = chronos::clk::now();
     if (lastTimeSync + after > now)
         return false;
 
@@ -169,20 +179,16 @@ bool Controller::sendTimeSyncMessage(long after)
 }
 
 
-void Controller::start(const PcmDevice& pcmDevice, const std::string& host, size_t port, int latency)
+void Controller::start()
 {
-    pcmDevice_ = pcmDevice;
-    latency_ = latency;
-    clientConnection_.reset(new ClientConnection(this, host, port));
+    clientConnection_ = make_unique<ClientConnection>(this, settings_.server.host, settings_.server.port);
     controllerThread_ = thread(&Controller::worker, this);
 }
 
 
-void Controller::run(const PcmDevice& pcmDevice, const std::string& host, size_t port, int latency)
+void Controller::run()
 {
-    pcmDevice_ = pcmDevice;
-    latency_ = latency;
-    clientConnection_.reset(new ClientConnection(this, host, port));
+    clientConnection_ = make_unique<ClientConnection>(this, settings_.server.host, settings_.server.port);
     worker();
     // controllerThread_ = thread(&Controller::worker, this);
 }
@@ -208,11 +214,11 @@ void Controller::worker()
             clientConnection_->start();
 
             string macAddress = clientConnection_->getMacAddress();
-            if (hostId_.empty())
-                hostId_ = ::getHostId(macAddress);
+            if (settings_.host_id.empty())
+                settings_.host_id = ::getHostId(macAddress);
 
             /// Say hello to the server
-            msg::Hello hello(macAddress, hostId_, instance_);
+            msg::Hello hello(macAddress, settings_.host_id, settings_.instance);
             clientConnection_->send(&hello);
 
             /// Do initial time sync with the server
@@ -221,8 +227,8 @@ void Controller::worker()
             {
                 if (async_exception_)
                 {
-                    LOG(DEBUG) << "Async exception: " << async_exception_->what() << "\n";
-                    throw SnapException(async_exception_->what());
+                    LOG(DEBUG) << "Async exception\n";
+                    std::rethrow_exception(async_exception_);
                 }
 
                 auto reply = clientConnection_->sendReq<msg::Time>(&timeReq, chronos::msec(2000));
@@ -237,19 +243,15 @@ void Controller::worker()
             /// Main loop
             while (active_)
             {
-                LOG(DEBUG) << "Main loop\n";
-                for (size_t n = 0; n < 10 && active_; ++n)
+                if (async_exception_)
                 {
-                    chronos::sleep(100);
-                    if (async_exception_)
-                    {
-                        LOG(DEBUG) << "Async exception: " << async_exception_->what() << "\n";
-                        throw SnapException(async_exception_->what());
-                    }
+                    LOG(DEBUG) << "Async exception\n";
+                    std::rethrow_exception(async_exception_);
                 }
 
-                if (sendTimeSyncMessage(5000))
+                if (sendTimeSyncMessage(1000ms))
                     LOG(DEBUG) << "time sync main loop\n";
+                this_thread::sleep_for(100ms);
             }
         }
         catch (const std::exception& e)
