@@ -93,7 +93,7 @@ void ControlSessionTcp::stop()
 void ControlSessionTcp::sendAsync(const std::string& message)
 {
     strand_.post([ this, self = shared_from_this(), message ]() {
-        messages_.emplace_back(message);
+        messages_.emplace_back(message + "\r\n");
         if (messages_.size() > 1)
         {
             LOG(DEBUG) << "TCP session outstanding async_writes: " << messages_.size() << "\n";
@@ -105,8 +105,7 @@ void ControlSessionTcp::sendAsync(const std::string& message)
 
 void ControlSessionTcp::send_next()
 {
-    auto message = messages_.front();
-    boost::asio::async_write(socket_, boost::asio::buffer(message + "\r\n"),
+    boost::asio::async_write(socket_, boost::asio::buffer(messages_.front()),
                              boost::asio::bind_executor(strand_, [ this, self = shared_from_this() ](std::error_code ec, std::size_t length) {
                                  messages_.pop_front();
                                  if (ec)
