@@ -128,13 +128,14 @@ void StreamSession::stop()
 
 void StreamSession::send_next()
 {
-    auto buffer = messages_.front();
+    auto& buffer = messages_.front();
+    buffer.on_air = true;
     boost::asio::async_write(socket_, buffer,
                              boost::asio::bind_executor(strand_, [ this, self = shared_from_this(), buffer ](boost::system::error_code ec, std::size_t length) {
                                  messages_.pop_front();
                                  if (ec)
                                  {
-                                     LOG(ERROR, LOG_TAG) << "StreamSession write error (msg lenght: " << length << "): " << ec.message() << "\n";
+                                     LOG(ERROR, LOG_TAG) << "StreamSession write error (msg length: " << length << "): " << ec.message() << "\n";
                                      messageReceiver_->onDisconnect(this);
                                      return;
                                  }
@@ -147,10 +148,22 @@ void StreamSession::send_next()
 void StreamSession::sendAsync(shared_const_buffer const_buf, bool send_now)
 {
     strand_.post([ this, self = shared_from_this(), const_buf, send_now ]() {
+        // delete PCM chunks that are older than the overall buffer duration
+        messages_.erase(std::remove_if(messages_.begin(), messages_.end(),
+                                       [this](const shared_const_buffer& buffer) {
+                                           const auto& msg = buffer.message();
+                                           if (!msg.is_pcm_chunk || buffer.on_air)
+                                               return false;
+                                           auto age = chronos::clk::now() - msg.rec_time;
+                                           return (age > std::chrono::milliseconds(bufferMs_) + 100ms);
+                                       }),
+                        messages_.end());
+
         if (send_now)
             messages_.push_front(const_buf);
         else
             messages_.push_back(const_buf);
+
         if (messages_.size() > 1)
         {
             LOG(DEBUG, LOG_TAG) << "outstanding async_write\n";
@@ -166,12 +179,8 @@ void StreamSession::sendAsync(msg::message_ptr message, bool send_now)
     if (!message)
         return;
 
-    // sendAsync(shared_const_buffer(*message), send_now);
-    tv t;
-    message->sent = t;
-    std::ostringstream oss;
-    message->serialize(oss);
-    sendAsync(shared_const_buffer(oss.str()), send_now);
+    // TODO: better set the timestamp in send_next for more accurate time sync
+    sendAsync(shared_const_buffer(*message), send_now);
 }
 
 
