@@ -49,27 +49,33 @@ public:
 
 
 // A reference-counted non-modifiable buffer class.
-// TODO: add overload for messages
 class shared_const_buffer
 {
-public:
-    // Construct from a std::string.
-    explicit shared_const_buffer(const std::string& data) : data_(new std::vector<char>(data.begin(), data.end())), buffer_(boost::asio::buffer(*data_))
+    struct Message
     {
+        std::vector<char> data;
+        bool is_pcm_chunk;
+        chronos::time_point_clk rec_time;
+    };
+
+public:
+    shared_const_buffer(msg::BaseMessage& message) : on_air(false)
+    {
+        tv t;
+        message.sent = t;
+        const msg::PcmChunk* pcm_chunk = dynamic_cast<const msg::PcmChunk*>(&message);
+        message_ = std::make_shared<Message>();
+        message_->is_pcm_chunk = (pcm_chunk != nullptr);
+        if (message_->is_pcm_chunk)
+            message_->rec_time = pcm_chunk->start();
+
+        std::ostringstream oss;
+        message.serialize(oss);
+        std::string s = oss.str();
+        message_->data = std::vector<char>(s.begin(), s.end());
+        buffer_ = boost::asio::buffer(message_->data);
     }
 
-    // // Construct from a message.
-    // explicit shared_const_buffer(const msg::BaseMessage& message)
-    // {
-    //     std::ostringstream oss;
-    //     message.serialize(oss);
-
-    //     data_ = std::shared_ptr<std::vector<char>>(new std::vector<char>(oss.str().begin(), oss.str().end()));
-    //     //std::make_shared<std::vector<char>>(oss.str().begin(), oss.str().end());
-    //     buffer_ = boost::asio::buffer(*data_);
-    // }
-
-
     // Implement the ConstBufferSequence requirements.
     typedef boost::asio::const_buffer value_type;
     typedef const boost::asio::const_buffer* const_iterator;
@@ -77,13 +83,21 @@ public:
     {
         return &buffer_;
     }
+
     const boost::asio::const_buffer* end() const
     {
         return &buffer_ + 1;
     }
 
+    const Message& message() const
+    {
+        return *message_;
+    }
+
+    bool on_air;
+
 private:
-    std::shared_ptr<std::vector<char>> data_;
+    std::shared_ptr<Message> message_;
     boost::asio::const_buffer buffer_;
 };
 
