@@ -44,6 +44,8 @@ LibrespotStream::LibrespotStream(PcmListener* pcmListener, boost::asio::io_conte
     string devicename = uri_.getQuery("devicename", "Snapcast");
     string onevent = uri_.getQuery("onevent", "");
     bool normalize = (uri_.getQuery("normalize", "false") == "true");
+    bool autoplay = (uri_.getQuery("autoplay", "false") == "true");
+    killall_ = (uri_.getQuery("killall", "true") == "true");
 
     if (username.empty() != password.empty())
         throw SnapException("missing parameter \"username\" or \"password\" (must provide both, or neither)");
@@ -60,6 +62,8 @@ LibrespotStream::LibrespotStream(PcmListener* pcmListener, boost::asio::io_conte
         params_ += " --onevent \"" + onevent + "\"";
     if (normalize)
         params_ += " --enable-volume-normalisation";
+    if (autoplay)
+        params_ += " --autoplay";
     params_ += " --verbose";
 
     if (uri_.query.find("username") != uri_.query.end())
@@ -87,8 +91,11 @@ void LibrespotStream::initExeAndPath(const std::string& filename)
         exe_ = exe_.substr(exe_.find_last_of("/") + 1);
     }
 
-    /// kill if it's already running
-    execGetOutput("killall " + exe_);
+    if (killall_)
+    {
+        /// kill if it's already running
+        execGetOutput("killall " + exe_);
+    }
 }
 
 
