@@ -61,7 +61,7 @@ class PcmListener
 public:
     virtual void onMetaChanged(const PcmStream* pcmStream) = 0;
     virtual void onStateChanged(const PcmStream* pcmStream, const ReaderState& state) = 0;
-    virtual void onChunkRead(const PcmStream* pcmStream, msg::PcmChunk* chunk, double duration) = 0;
+    virtual void onChunkRead(const PcmStream* pcmStream, std::shared_ptr<msg::PcmChunk> chunk, double duration) = 0;
     virtual void onResync(const PcmStream* pcmStream, double ms) = 0;
 };
 
@@ -83,7 +83,7 @@ public:
     virtual void stop();
 
     /// Implementation of EncoderListener::onChunkEncoded
-    void onChunkEncoded(const encoder::Encoder* encoder, msg::PcmChunk* chunk, double duration) override;
+    void onChunkEncoded(const encoder::Encoder* encoder, std::shared_ptr<msg::PcmChunk> chunk, double duration) override;
     virtual std::shared_ptr<msg::CodecHeader> getHeader();
 
     virtual const StreamUri& getUri() const;
@@ -103,7 +103,7 @@ protected:
 
     void setState(const ReaderState& newState);
 
-    timeval tvEncodedChunk_;
+    std::chrono::time_point<std::chrono::steady_clock> tvEncodedChunk_;
     PcmListener* pcmListener_;
     StreamUri uri_;
     SampleFormat sampleFormat_;
